@@ -1,21 +1,21 @@
 use strict;
 use warnings;
-package Dist::Zilla::Plugin::DynamicPrereqs; # git description: v0.018-8-g0a19d4a
+package Dist::Zilla::Plugin::DynamicPrereqs; # git description: v0.021-7-gbfd1917
+# vim: set ts=8 sts=4 sw=4 tw=115 et :
 # ABSTRACT: Specify dynamic (user-side) prerequisites for your distribution
 # KEYWORDS: plugin distribution metadata MYMETA prerequisites Makefile.PL dynamic
-# vim: set ts=8 sts=4 sw=4 tw=115 et :
 
-our $VERSION = '0.019';
+our $VERSION = '0.022';
 
 use Moose;
 with
-    'Dist::Zilla::Role::InstallTool',
+    'Dist::Zilla::Role::FileMunger',
     'Dist::Zilla::Role::MetaProvider',
     'Dist::Zilla::Role::PrereqSource',
     'Dist::Zilla::Role::AfterBuild',
     'Dist::Zilla::Role::TextTemplate',
 ;
-use List::Util 1.33 qw(first notall any);
+use List::Util 1.45 qw(first notall any uniq);
 use Module::Runtime 'module_notional_filename';
 use Try::Tiny;
 use Path::Tiny;
@@ -95,6 +95,10 @@ sub BUILD
 {
     my ($self, $args) = @_;
 
+    $self->log_fatal('[MakeMaker::Awesome] must be at least version 0.19 to be used with [DynamicPrereqs]')
+        if $INC{module_notional_filename('Dist::Zilla::Plugin::MakeMaker::Awesome')}
+            and not eval { Dist::Zilla::Plugin::MakeMaker::Awesome->VERSION('0.19') };
+
     my %extra_args = %$args;
     delete @extra_args{ map { $_->name } $self->meta->get_all_attributes };
     if (my @keys = keys %extra_args)
@@ -104,6 +108,19 @@ sub BUILD
     }
 }
 
+around dump_config => sub
+{
+    my ($orig, $self) = @_;
+    my $config = $self->$orig;
+
+    my $data = {
+        blessed($self) ne __PACKAGE__ ? ( version => $VERSION ) : (),
+    };
+    $config->{+__PACKAGE__} = $data if keys %$data;
+
+    return $config;
+};
+
 sub metadata { return +{ dynamic_config => 1 } }
 
 sub after_build
@@ -116,79 +133,101 @@ sub after_build
 # track which subs have already been included by some other instance
 my %included_subs;
 
-sub setup_installer
+sub munge_files
 {
     my $self = shift;
 
-    $self->log_fatal('[MakeMaker::Awesome] must be at least version 0.19 to be used with [DynamicPrereqs]')
-        if $INC{module_notional_filename('Dist::Zilla::Plugin::MakeMaker::Awesome')}
-            and not eval { Dist::Zilla::Plugin::MakeMaker::Awesome->VERSION('0.19') };
-
     my $file = first { $_->name eq 'Makefile.PL' } @{$self->zilla->files};
-    $self->log_fatal('No Makefile.PL found!') if not $file;
+    $self->log_fatal('No Makefile.PL found! Is [MakeMaker] at least version 5.022?') if not $file;
 
     my $content = $file->content;
 
     $self->log_debug('Inserting dynamic prereq into Makefile.PL...');
 
+    # we insert our code just *before* this bit in Makefile.PL
+    my $insertion_breadcrumb = "\n\nunless ( eval { ExtUtils::MakeMaker";
+
     # insert after declarations for BOTH %WriteMakefileArgs, %FallbackPrereqs.
     # TODO: if marker cannot be found, fall back to looking for just
     # %WriteMakefileArgs -- this requires modifying the content too.
     $self->log_fatal('failed to find position in Makefile.PL to munge!')
-        if $content !~ m'^my %FallbackPrereqs = \((?:\n[^;]+^)?\);$'mg;
+        if $content !~ m/\Q$insertion_breadcrumb/mg;
 
-    my $pos = pos($content);
+    my $pos = pos($content) - length($insertion_breadcrumb);
 
     my $code = join("\n", $self->raw);
     if (my $conditions = join(' && ', $self->conditions))
     {
         $code = "if ($conditions) {\n"
             . $code . "\n"
-            . '}';
+            . '}' . "\n";
     }
 
     $content = substr($content, 0, $pos)
-        . "\n\n"
-        . "# inserted by " . blessed($self) . ' ' . $self->VERSION . "\n"
-        . $code
+        . "\n"
+        . $self->_header . "\n"
+        . $code . "\n"
         . substr($content, $pos);
 
     $content =~ s/\n+\z/\n/;
 
-    if (my @include_subs = grep { not exists $included_subs{$_} } $self->_all_required_subs)
-    {
-        $content .= "\n# inserted by " . blessed($self) . ' ' . $self->VERSION
-            if not keys %included_subs;
-
-        if (my @missing_subs = grep { !-f path($self->_include_sub_root, $_) } @include_subs)
-        {
-            $self->log_fatal(
-                @missing_subs > 1
-                    ? [ 'no definitions available for subs %s!', join(', ', map { "'" . $_  ."'" } @missing_subs) ]
-                    : [ 'no definition available for sub \'%s\'!', $missing_subs[0] ]
-            );
-        }
-
-        # On consultation with ribasushi I agree that we cannot let authors
-        # use some sub definitions without copious danger tape.
-        $self->_warn_include_subs(@include_subs);
-
-        my @sub_definitions = map { path($self->_include_sub_root, $_)->slurp_utf8 } @include_subs;
-        $content .= "\n"
-            . $self->fill_in_string(
-                join("\n", @sub_definitions),
-                {
-                    dist => \($self->zilla),
-                    plugin => \$self,
-                },
-            );
-        @included_subs{@include_subs} = (() x @include_subs);
-    }
+    $content .= $self->_sub_definitions;
 
     $file->content($content);
     return;
 }
 
+has _header => (
+    is => 'ro', isa => 'Str',
+    lazy => 1,
+    default => sub {
+        my $self = shift;
+        '# inserted by ' . blessed($self) . ' ' . $self->VERSION;
+    },
+);
+
+has _sub_definitions => (
+    is => 'ro', isa => 'Str',
+    lazy => 1, builder => '_build__sub_definitions',
+);
+
+sub _build__sub_definitions
+{
+    my $self = shift;
+
+    my @include_subs = grep { not exists $included_subs{$_} } $self->_all_required_subs;
+    return '' if not @include_subs;
+
+    my $content;
+    $content .= "\n" . $self->_header if not keys %included_subs;
+
+    if (my @missing_subs = grep { !-f path($self->_include_sub_root, $_) } @include_subs)
+    {
+        $self->log_fatal(
+            @missing_subs > 1
+                ? [ 'no definitions available for subs %s!', join(', ', map { "'" . $_  ."'" } @missing_subs) ]
+                : [ 'no definition available for sub \'%s\'!', $missing_subs[0] ]
+        );
+    }
+
+    # On consultation with ribasushi I agree that we cannot let authors
+    # use some sub definitions without copious danger tape.
+    $self->_warn_include_subs(@include_subs);
+
+    my @sub_definitions = map { path($self->_include_sub_root, $_)->slurp_utf8 } @include_subs;
+    $content .= "\n"
+        . $self->fill_in_string(
+            join("\n", @sub_definitions),
+            {
+                dist => \($self->zilla),
+                plugin => \$self,
+            },
+        );
+    @included_subs{@include_subs} = (() x @include_subs);
+
+    return $content;
+}
+
 my %sub_prereqs = (
     can_xs => {
         'ExtUtils::CBuilder' => '0.27',
@@ -260,7 +299,7 @@ has _all_required_subs => (
                 any { /\b$sub_name\b\(/ } @code
             } map { $_->basename } path($self->_include_sub_root)->children;
 
-        [ sort($self->_all_required_subs_for(_uniq(
+        [ sort($self->_all_required_subs_for(uniq(
             $self->include_subs, @subs_in_code,
         ))) ];
     },
@@ -297,8 +336,6 @@ sub _warn_include_subs
         foreach grep { exists $warn_include_sub{$_} } @include_subs;
 }
 
-sub _uniq { keys %{ +{ map { $_ => undef } @_ } } }
-
 __PACKAGE__->meta->make_immutable;
 
 __END__
@@ -313,7 +350,7 @@ Dist::Zilla::Plugin::DynamicPrereqs - Specify dynamic (user-side) prerequisites
 
 =head1 VERSION
 
-version 0.019
+version 0.022
 
 =head1 SYNOPSIS
 
@@ -331,7 +368,6 @@ or:
     -delimiter = |
     -raw = |test_requires('Devel::Cover')
     -raw = |    if $ENV{EXTENDED_TESTING} or is_smoker();
-    -include_sub = is_smoker
 
 or:
 
@@ -392,7 +428,7 @@ C<[Prereqs / ConfigureRequires]> in your F<dist.ini>).
 
 C<-body> first became available in version 0.018.
 
-=for Pod::Coverage mvp_multivalue_args mvp_aliases BUILD metadata after_build setup_installer register_prereqs
+=for Pod::Coverage mvp_multivalue_args mvp_aliases BUILD metadata after_build munge_files register_prereqs
 
 =head2 C<-delimiter>
 
@@ -436,7 +472,8 @@ statement, and the C<-raw>/C<-body> lines are contained as the body of the block
     -condition = $] > '5.020'
     -body = requires('1.003000')
 
-results in the F<Makefile.PL> snippet:
+results in the F<Makefile.PL> snippet (note that whitespace is not added, in
+case this affects the parsing:
 
     if ($] > '5.020') {
     requires('Role::Tiny', '1.003000')
@@ -474,7 +511,7 @@ C<parse_args()> - returns the hashref of options that were passed as arguments t
 
 =item *
 
-C<can_xs()> - Secondary compile testing via ExtUtils::CBuilder
+C<can_xs()> - Secondary compile testing via L<ExtUtils::CBuilder>
 
 =item *
 
@@ -486,11 +523,11 @@ C<can_run()> - check if we can run some command
 
 =item *
 
-C<can_use($module, $version (optional))> - checks if a module (optionally, at a specified version) can be loaded. (If you don't want to load the module, you should use C<< has_module >>, see below.)
+C<can_use($module [, $version ])> - checks if a module (optionally, at a specified version) can be loaded. (If you don't want to load the module, you should use C<< has_module >>, see below.)
 
 =item *
 
-C<has_module($module, $version_or_range (optional))> - checks if a module (optionally, at a specified version or matching a L<version range|CPAN::Meta::Spec/version_ranges>) is available in C<%INC>. Does not load the module, so is safe to use with modules that have side effects when loaded.  When passed a second argument, returns true or false; otherwise, returns undef or the module's C<$VERSION>. (Current API available since version 0.015.)
+C<has_module($module [, $version_or_range ])> - checks if a module (optionally, at a specified version or matching a L<version range|CPAN::Meta::Spec/version_ranges>) is available in C<%INC>. Does not load the module, so is safe to use with modules that have side effects when loaded.  When passed a second argument, returns true or false; otherwise, returns undef or the module's C<$VERSION>. (Current API available since version 0.015.)
 
 =item *
 
@@ -518,29 +555,31 @@ C<maybe_command> - actually a monkeypatch to C<< MM->maybe_command >> (please ke
 
 =item *
 
-C<runtime_requires($module, $version (optional))> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
+C<runtime_requires($module [, $version ])> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
 
 =item *
 
-C<requires($module, $version (optional))> - alias for C<runtime_requires>. Added in 0.016.
+C<requires($module [, $version ])> - alias for C<runtime_requires>. Added in 0.016.
 
 =item *
 
-C<build_requires($module, $version (optional))> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
+C<build_requires($module [, $version ])> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
 
 =item *
 
-C<test_requires($module, $version (optional))> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
+C<test_requires($module [, $version ])> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
 
 =back
 
 =head1 WARNING: INCOMPLETE SUBROUTINE IMPLEMENTATIONS!
 
 The implementations for some subroutines (in particular, C<can_xs>, C<can_cc>
-and C<can_run> are still incomplete, incompatible with some architectures and
+and C<can_run> are still works in progress, incompatible with some architectures and
 cannot yet be considered a suitable generic solution. Until we are more
 confident in their implementations, a warning will be printed upon use, and
-their use B<is not advised> without prior consultation with the author.
+their use B<is not advised> without prior consultation with the author and
+other members of the Perl Toolchain Gang
+(see L<C<#toolchain> on C<irc.perl.org>|irc://irc.perl.org/#toolchain>).
 
 =head1 WARNING: UNSTABLE API!
 
@@ -555,25 +594,27 @@ upstream users, but I obviously cannot do this for DarkPAN users. Regardless,
 please contact me (see below) and I will keep you directly advised of
 interface changes.
 
-Future options may include:
+Future planned features:
+
+=for stopwords CPANFile
 
 =over 4
 
 =item *
 
-C<-phase> the phase in which subsequently-specified module/version pairs will be added
+better compiler detection and conditional XS code inclusion
 
 =item *
 
-C<-runtime> a module and version that is added to runtime prereqs should the C<-condition> be satisfied
+interoperability with the L<[CPANFile]|Dist::Zilla::Plugin::CPANFile> plugin (generation of dynamic prerequisites into a F<cpanfile>)
 
 =item *
 
-C<-test> a module and version that is added to test prereqs should the C<-condition> be satisfied
+something like C<is_perl_at_least('5.008001')> for testing C<$]>
 
 =item *
 
-C<-build> a module and version that is added to build prereqs should the C<-condition> be satisfied
+inlining of sub content for some checks, to allow constant folding (e.g. C<$^O> and C<$]> checks)
 
 =back
 
@@ -612,7 +653,7 @@ There is also a mailing list available for users of this distribution, at
 L<http://dzil.org/#mailing-list>.
 
 There is also an irc channel available for users of this distribution, at
-L<irc://irc.perl.org/#distzilla>.
+L<C<#distzilla> on C<irc.perl.org>|irc://irc.perl.org/#distzilla>.
 
 I am also usually active on irc, as 'ether' at C<irc.perl.org>.
 
